@@ -1,8 +1,7 @@
 import string
+from typing import List, Optional, Union
+
 import numpy as np
-
-from typing import Union, List, Optional
-
 from instancelib.instances.text import TextInstanceProvider
 from text_explainability.default import Readable
 
