@@ -26,6 +26,7 @@ def oneway_dictionary_mapping(instance: TextInstance,
 
     # Get all options
     options = {i: dictionary[a] for i, a in enumerate(tokenized) if a in dictionary.keys()}
+    option_keys = list(options.keys())
     all_options = list(itertools.product(*options.values()))
 
     # Pick up to N random replacements and apply them
@@ -33,7 +34,7 @@ def oneway_dictionary_mapping(instance: TextInstance,
         current_option = all_options[idx]
         new_tokenized = copy.deepcopy(tokenized)
         for i, option in enumerate(current_option):
-            new_tokenized[options.keys()[i]] = option
+            new_tokenized[option_keys[i]] = option
         if tokenized != new_tokenized:
             yield detokenizer(new_tokenized), label_from, label_to
 
