@@ -1,9 +1,10 @@
 """Sensitivity testing, for fairness and robustness."""
 
+from typing import Iterator, List, Optional, Tuple, Union
 
-from typing import Iterator, Union, List, Optional, Tuple
-
-from instancelib.analysis.base import label_metrics, BinaryModelMetrics, MulticlassModelMetrics
+import numpy as np
+from instancelib.analysis.base import (BinaryModelMetrics,
+                                       MulticlassModelMetrics, label_metrics)
 from instancelib.environment.text import TextEnvironment
 from instancelib.instances.base import InstanceProvider
 from instancelib.instances.text import MemoryTextInstance, TextInstanceProvider
@@ -11,7 +12,9 @@ from instancelib.labels.memory import MemoryLabelProvider
 from instancelib.machinelearning.base import AbstractClassifier
 from instancelib.typehints import LT
 
-from text_sensitivity.data.random.string import RandomString, combine_generators
+from text_sensitivity.data.generate import from_pattern
+from text_sensitivity.data.random.string import (RandomString,
+                                                 combine_generators)
 from text_sensitivity.perturbation.base import Perturbation
 
 
@@ -188,3 +191,68 @@ def input_space_robustness(model: AbstractClassifier,
     success = 1.0 if len(instances) == 0 else success / len(instances)
 
     return success, failures
+
+
+def invariance(pattern: str,
+               model: AbstractClassifier,
+               expectation: Optional[LT] = None,
+               **kwargs,
+               ) -> Tuple[float, List[MemoryTextInstance], List[MemoryTextInstance], MemoryLabelProvider]:
+    """Test for the failure rate under invariance.
+
+    Args:
+        pattern (str): String pattern to generate examples from.
+        model (AbstractClassifier): Model to test.
+        expectation (Optional[LT], optional): Expected outcome values. Defaults to None.
+        **kwargs: Optional arguments passed onto the `data.generate.from_pattern()` function.
+
+    Returns:
+        Tuple[float, List[MemoryTextInstance], List[MemoryTextInstance], MemoryLabelProvider]: Failure rate
+            (percentage), list of correctly invariant instances, list of wrong instances (variant), and perturbation 
+            labels for subset selection.
+    """
+    instances, _ = from_pattern(pattern, **kwargs)
+    predictions = model.predict(instances)
+
+    if expectation is None:
+        if len(predictions) == 0:
+            return 0.0, [], []
+        expectation = predictions[0][-1]
+    if not isinstance(expectation, frozenset):
+        expectation = frozenset({expectation})
+
+    correct = [instances[id] for id, label in predictions if label == expectation]
+    wrong = [instances[id] for id, label in predictions if label != expectation]
+
+    return len(wrong) / len(predictions), correct, wrong
+
+
+def mean_score(pattern: str,
+               model: AbstractClassifier,
+               selected_label: Optional[LT] = None,
+               **kwargs) -> Tuple[float, List[Tuple[MemoryTextInstance, float]]]:
+    """Calculate mean (probability) score for a given label, for data generated from a pattern.
+
+    Args:
+        pattern (str): 
+        model (AbstractClassifier): Model to generate scores from.
+        selected_label (Optional[LT], optional): Label name to select. If None is replaced by the first label.
+            Defaults to None.
+
+    Returns:
+        Tuple[float, List[Tuple[LT, float]]]: Mean score for the selected label, generated instances and label scores.
+    """
+    instances, _ = from_pattern(pattern, **kwargs)
+    predictions = model.predict_proba_provider(instances)
+
+    if selected_label is None:
+        if len(predictions) == 0:
+            return
+        selected_label = list(model.encoder.labelset)[0]
+    if isinstance(selected_label, frozenset):
+        selected_label = list(selected_label)[0]
+
+    predictions_by_label = [(instances[id], [proba for label, proba in list(probas) if label == selected_label][0])
+                            for id, probas in predictions]
+
+    return np.mean([p for id, p in predictions_by_label]), predictions_by_label
